<?php

namespace HCM\FPA\Calculos\Contabilizacao;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\FPA\Repository\ContabilizacaoMensalRepository;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\FPA\Services\CalculoFolhaService;
use HCM\FPA\Entity\FparesumoctblEntity;
use HCM\Util\FPA;
use \Setforms2\Shared\ORM\DateTime;
use HCM\FPA\Controllers\LogRetrocessoCalculoController;

set_time_limit(0);
ini_set('memory_limit', '2048M');

class PercorreContabilizacao {
    
    protected $entityManager;
    protected $hcmParameters;
    const TP_TRANSF_LEGAL = '1';
    const TP_TRANSF_GEREN = '2';

    private $_nrorg;
    private $_nrOrgPadrao;
    private $_nrEstruturaTrab;
    private $_nrTpModalidadeCalc;
    private $_nrOcorrenciaCalc;
    private $_idFormaContab;
    private $_mes_competencia;
    private $_tipo_movimento;
    private $_nr_tipo_hierarquia;
    private $_simulacao;
    private $_retrocesso;
    private $_dsOrdem;
    private $_dsInicio;
    private $_dsFim;
    private $_vinculom;
    private $_hierarquia_org;
    private $_tipo_hierarquia;
    private $_movimentacao;
    private $_calculo_folha;
    private $_formulario;
    private $_evento;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, CalculoFolhaService $calculoFolhaService, ContabilizacaoMensalRepository $contabilizacaoMensalRepository, LogRetrocessoCalculoController $logRetrocessoCalculoController) {
        $this->entityManager      = $entityManager;
        $this->hcmParameters      = $hcmParameters;
        $this->gpe_Vinculom = Repositories::GPE_VINCULOM;
        
        $this->_nrorg = $hcmParameters->getNrOrgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrOrgPadrao();
        $this->_mes_competencia = $hcmParameters->getDtCompetencia();
        $this->_nrTpModalidadeCalc = $hcmParameters->getNrModalidadeCalculo();
        
        $this->calculoFolhaService = $calculoFolhaService;
        $this->contabilizacaoMensalRepository = $contabilizacaoMensalRepository;
        $this->_vinculom = new GpevinculomEntity($entityManager, $hcmParameters);

        $this->_tipo_hierarquia = new TipohierarqorgEntity($entityManager, $hcmParameters);
        $this->_resumoContabil = new FparesumoctblEntity($entityManager, $hcmParameters);

        $this->logRetrocessoCalculoController = $logRetrocessoCalculoController;
        
        $this->contabBenef = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->
            buscavalor($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getDtCompetencia(), 'CONTAB_BENEF_RESC');
    }
    
    public function percorre($dataSet) {
            
        // Caso o processamento seja o retrocesso deve-se gravar o log da operação
        if($dataSet['TP_PROCESSAMENTO'] == "RETROCESSO") $this->logRetrocessoCalculoController->persistirLogRetrocesso('CALC_CONTAB', $dataSet);
            
        $this->_buscaDadosFormulario($dataSet);
        $lista_calculo_folha = array();
        if ($this->_retrocesso === false) {
            if (!$this->_nrOcorrenciaCalc) {
                $listaNrCalculo = $this->calculoFolhaService->carregaListaNrCalculoFolhaTodasOcorrencias(
                    $this->_nrorg, $this->_tipo_movimento, $this->_mes_competencia, $this->_nrTpModalidadeCalc
                );
                foreach ($listaNrCalculo as $nrCalculo) {
                    $lista_calculo_folha[] = $nrCalculo;
                }
            } else {
                $arrayOcorrencias = explode(',',$this->_nrOcorrenciaCalc);
                foreach ($arrayOcorrencias as $nrocorrencia) {
                    $calculo_folha = $this->calculoFolhaService->retornaCalculoFolha(
                        $this->_nrorg, $this->_tipo_movimento, $this->_mes_competencia, $this->_nrTpModalidadeCalc, $nrocorrencia
                    );

                    foreach ($calculo_folha as $calc_folha) {
                        $lista_calculo_folha[] = $calc_folha->getNrcalculofolha();
                    }
                }
            }
        }

        $contabiliza = new Contabilizacao($this->entityManager, $this->hcmParameters, $this->_simulacao, $dataSet);
        
        $this->_verificaEstruturasVinculos();

        if(!$this->_verificaAssociacaoContabil($lista_calculo_folha)) {
            return array('msg' => $this->msg);
        }
        $vinculos = $this->_vinculom->retornaVinculosAtivosMesCompetencia(
            $this->_nrorg, $this->_mes_competencia, $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, true,
            $this->_nrEstruturaTrab
        );
        
        $vinculos_resc = $this->_vinculom->retornaVinculosRescisaoComplementarCompetencia($this->_nrorg, $this->_mes_competencia,
            $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, null, null, $this->_nrEstruturaTrab);

        if ($vinculos_resc) {
            $vinculos = array_merge($vinculos, $vinculos_resc);
        }
        
        $vinculos_plr = $this->_vinculom->retornaVinculosRescindidosComPlrCompetencia($this->_nrorg, $this->_mes_competencia,
            $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, null, null, $this->_nrEstruturaTrab);
        
        if ($vinculos_plr) {
            $vinculos = array_merge($vinculos, $vinculos_plr);
        }
        
        if ($this->contabBenef == 1) {
            $vinculos_beneficio = $this->_vinculom->retornaVinculosRescindidosComBeneficioCompetencia($this->_nrorg, $this->_mes_competencia,
                $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, null, null, $this->_nrEstruturaTrab);
            
            if ($vinculos_beneficio) {
                $vinculos = array_merge($vinculos, $vinculos_beneficio);
            }
        }
        
        $estruturas = array();
        foreach ($vinculos as $key => $vinculo) {
            if ($this->_retrocesso === false) {
                try {
                    $result_parcial = $contabiliza->contabilizar($vinculo, $lista_calculo_folha, $this->_nr_tipo_hierarquia);
                    if($this->_simulacao === false && (sizeof($vinculos)==$key+1 || $vinculos[$key+1]->historico->getNrestrutgeren() != $vinculo->historico->getNrestrutgeren())){
                        if ($this->_idFormaContab == 'C' || $this->_idFormaContab == 'T') 
                            $contabiliza->persistResumoContabil();
                        if ($this->_idFormaContab == 'D' || $this->_idFormaContab == 'T') 
                            $contabiliza->persistResumoDro();
                    }
                } catch(Exception $e) {
                    throw $e;
                }
            } else {
                if (!in_array($contabiliza->_determinaNrEstruturaSuperior($vinculo), $estruturas)) {
                    $estruturas[] = $contabiliza->_determinaNrEstruturaSuperior($vinculo);  
                }
            }
        }
            
        if ($this->_retrocesso) {
            $retrocede = false;
            foreach ($estruturas as $nr_estrutura_sup) {
                if($dataSet['IDFORMACONTAB']!='D')
                    $aux = $this->contabilizacaoMensalRepository->retrocedeResumoCTBL($nr_estrutura_sup, $this->_tipo_movimento);
                if($dataSet['IDFORMACONTAB']!='C')
                    $aux = $this->contabilizacaoMensalRepository->retrocedeResumoDRO($nr_estrutura_sup, $this->_tipo_movimento);
                if($aux) $retrocede = true;
            }
            return array($retrocede);
        }
        
        if (($this->_retrocesso === false) && ($this->_simulacao === false)) {
            if ($this->_idFormaContab == 'C' || $this->_idFormaContab == 'T') {
                $contabiliza->persistResumoContabil();
            }
            if ($this->_idFormaContab == 'D' || $this->_idFormaContab == 'T') {
                $contabiliza->persistResumoDro();
            }
        }
        
        $demonstrativo = array(false);
        if(($this->_retrocesso === false)){
            $demonstrativo = $contabiliza->preparaDemonstrativo();
        }
        
        return $demonstrativo;
    }
    
    public function percorreLite($dataSet) {
            
        $this->_buscaDadosFormulario($dataSet);
        $lista_calculo_folha = array();
        
        if ($this->_retrocesso === false) {
            if (!$this->_nrOcorrenciaCalc) {
                $listaNrCalculo = $this->calculoFolhaService->carregaListaNrCalculoFolhaTodasOcorrencias(
                    $this->_nrorg, $this->_tipo_movimento, $this->_mes_competencia, $this->_nrTpModalidadeCalc
                );
                foreach ($listaNrCalculo as $nrCalculo) {
                    $lista_calculo_folha[] = $nrCalculo;
                }
            } else {
                $arrayOcorrencias = explode(',',$this->_nrOcorrenciaCalc);
                foreach ($arrayOcorrencias as $nrocorrencia) {
                    $calculo_folha = $this->calculoFolhaService->retornaCalculoFolha(
                        $this->_nrorg, $this->_tipo_movimento, $this->_mes_competencia, $this->_nrTpModalidadeCalc, $nrocorrencia
                    );

                    foreach ($calculo_folha as $calc_folha) {
                        $lista_calculo_folha[] = $calc_folha->getNrcalculofolha();
                    }
                }
            }
        }
        
        $this->_verificaEstruturasVinculos();

        if(!$this->_verificaAssociacaoContabil($lista_calculo_folha)) {
            return array('msg' => $this->msg);
        }
        
        $dadosContabilizacao = $this->contabilizacaoMensalRepository->getContabilizacao(
            $this->_nrorg, $this->_mes_competencia, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim, $this->_tipo_movimento, $this->_nrTpModalidadeCalc, $this->_nrOcorrenciaCalc
        );
        
        $contabiliza = new Contabilizacao($this->entityManager, $this->hcmParameters, $this->_simulacao, $dataSet);
        
        try {
            $result_parcial = $contabiliza->contabilizarLite($dadosContabilizacao);
            if ($this->_simulacao === false && count($dadosContabilizacao) > 0) {
                if ($this->_idFormaContab == 'C' || $this->_idFormaContab == 'T') 
                    $contabiliza->persistResumoContabil();
            }
        } catch(Exception $e) {
            throw $e;
        }
        
        if (($this->_retrocesso === false) && ($this->_simulacao === false)) {
            if ($this->_idFormaContab == 'C' || $this->_idFormaContab == 'T') {
                $contabiliza->persistResumoContabil();
            }
        }
        
        $demonstrativo = array(false);
        if(($this->_retrocesso === false)){
            $demonstrativo = $contabiliza->preparaDemonstrativo();
        }
        
        return $demonstrativo;
    }

    private function _buscaDadosFormulario($dataSet){
        $this->_formulario = $dataSet;
        $this->_dsOrdem = $dataSet['DSORDEM'];
        $this->_dsInicio = $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_nr_tipo_hierarquia = $dataSet['NRHIERARQORG'];
        $this->_tipo_movimento = $dataSet['NRTIPOMOVIMENT'];
        $this->_nrOcorrenciaCalc = $dataSet['NROCORRECAL'];
        $this->_idFormaContab = $dataSet['IDFORMACONTAB'];
        $tp_processamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch ($tp_processamento){
        case FPA::PROC_SIMULACAO :
            $this->_simulacao = true;
            $this->_retrocesso = false;
            break;
        case FPA::PROC_CALCULO :
            $this->_simulacao = false;
            $this->_retrocesso = false;
            break;
        case FPA::PROC_RETROCESSO :
            $this->_retrocesso = true; break;
        default:
            throw new \Exception('Tipo de processo '.$tp_processamento.' invalido');
        }
    }
    
    private function _verificaEstruturasVinculos(){

        $nrtpestrutlegal = FPA::getTipoEstruturaLegalOrg();
        $nrtpestrutgeren = FPA::getTipoEstruturaGerencialOrg();
        
        $estrutlegal = $this->_vinculom->getVinculosEstrutLegalDifereMovimentacao($this->_nrorg, $this->_mes_competencia, self::TP_TRANSF_LEGAL, $nrtpestrutlegal, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);
        $estrutgeren = $this->_vinculom->getVinculosEstrutGerenDifereMovimentacao($this->_nrorg, $this->_mes_competencia, self::TP_TRANSF_GEREN, $nrtpestrutgeren, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);

        if ($estrutlegal || $estrutgeren) {
            $msg = "<b>Erros de movimentação </b><br>";
            $msg .= "<b>----------------------</b><br>";

            if($estrutlegal) {
                $msg .= "<b>Vinculos com estrutura legal do histórico diferente da movimentação legal: </b><br>";
                /** @var GpevinculomEntity $vinculo */
                foreach ($estrutlegal as $vinculo) {
                    $msg .= "&nbsp;&nbsp;&nbsp;" . $vinculo->getNrvinculom() . " - " . $vinculo->getNmvinculom() . "<br>";
                }
            }

            if($estrutgeren) {
                $msg .= "<b>Vinculos com estrutura gerencial do histórico diferente da movimentação gerencial: </b><br>";
                /** @var GpevinculomEntity $vinculo */
                foreach ($estrutgeren as $vinculo) {
                    $msg .= "&nbsp;&nbsp;&nbsp;" . $vinculo->getNrvinculom() . " - " . $vinculo->getNmvinculom() . "<br>";
                }
            }

            throw new \Exception($msg);
        }

    }

    private function _verificaAssociacaoContabil($lista_calculo_folha){

        $erros = array();

        foreach ($lista_calculo_folha as $nr_calculo_folha) {
            $eventos = $this->_resumoContabil->getEventoSemAssociacaoPorSelecaoVinculos($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, $nr_calculo_folha, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);

            if ($eventos) {
               $erros = array_merge($erros, $eventos);
            }
        }
        
        $estrutura = '';
        if (!empty($erros)) {
            $this->msg = '';
            foreach ($erros as $evento) {
                if($estrutura !== $evento['nmestruturam']){
                    $this->msg .= PHP_EOL."<b>Erros - Eventos sem  Associação Contábil para a estrutura legal</b><br>".$evento['nmestruturam'].PHP_EOL;
                }
                $this->msg .= PHP_EOL.$evento['nreventom'] . " - " . $evento['nmeventoh'] .PHP_EOL;
                $estrutura = $evento['nmestruturam'];
            }
            return false;
        }
        return true;
    }
}