<?php

namespace HCM\FPA\Calculos\Provisao;

use HCM\FPA\Calculos\Geral\Calculo;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Entity\FpatipoincideEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaPtregraEntity;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\FPA\Entity\FpaferiasEntity;
use HCM\FPA\Entity\FpaTpdemissaoEntity;
use HCM\FPA\Entity\FpatipoferiasmEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpevinculohEntity;
use HCM\FPA\Entity\FpaParamfolhahEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaFormcalculoEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use Teknisa\Model\Gestaopessoas\Repositories\FpaValorreferenciam;
use HCM\FPA\Calculos\Geral\FGTS;
use HCM\FPA\Calculos\Geral\INSS;
use HCM\FPA\Calculos\Geral\IRRF;


class CalculoProvisaoOtimizada extends Calculo {

    protected $valorComplemento;

    public function calculaProvisaoOtimizada($vinculo, $tp_calc_mov, $contador){
        
        $this->_inicializaVariaveis($vinculo, $tp_calc_mov);
        
        echo "inicializaVariaveis".PHP_EOL;
        
        if ($tp_calc_mov->retornaItensCalculo($vinculo) === false) {
            return;
        }
        
        try {
            $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
            $this->_itens_calculo_principal =  $obj_tp_calc_mov->getInstancia();
            $this->_itens_calculo_principal->setNrTipoCalculo($this->_nr_tipo_calculo);
        } catch(Exception $e) {
            $message = $e->getMessage();
            $message .= ' - CALCULO.PHP';
            MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
        }
        
        $this->_controleLiquidoParcial();
        
        echo "_controleLiquidoParcial".PHP_EOL;
        
        $this->_inicializaVariaveisRegra();
        
        echo "_inicializaVariaveisRegra".PHP_EOL;

        $this->_calculo_regra->geraVariaveis();
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_INICIO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        
        echo "PtInicio".PHP_EOL;
        
        $this->_verificaTrabalhoEfetivo($vinculo);
        
        echo "_verificaTrabalhoEfetivo".PHP_EOL;
        
        if($this->_sit_func_resumo[14] >= 15){
            MsgCalculo::getInstance()->adicionaMensagem('Vinculo aposentado por invalidez.');
            return;
        }
        
        $this->_executaProvisaoOtimizada($vinculo, $tp_calc_mov);
        
        echo "_executaProvisaoOtimizada".PHP_EOL;
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_FINAL, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        
        echo "ptregraFinal".PHP_EOL;
        
        $lista = null;
        
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            return $lista;
        }
        if ($this->_simulacao === false) {
            $tp_calc_mov->persisteItensCalculo();
            return $lista;
        } else {
            $lista['SIMULACAO'] = $tp_calc_mov->retornaListaItensCalculo(
                $this->_mes_competencia, $this->_eventom, $vinculo
            );
            //Vinculo
            if(count($lista) > 0){
                $lista['SIMULACAO'][0][] = array(
                    'MESOCORRENCIA' => $vinculo->getNrvinculom(),
                    'TIPOEVENTO' => $vinculo->getNmvinculom(),
                    'NRFILHO' => $vinculo->getNrvinculom(),
                    'NREVENTOM' => 'Matricula: '.$vinculo->getCdmatricula(),
                    'NMEVENTOH' => 'Salario: ' .number_format($this->_salario, 2, ',','.'),
                    'UNIDREFEEVEN' => 'Admissão: '.substr($vinculo->getDtadmissaovinc(), 0, 10),
                    'VRREFFOLHA' => null,
                    'IDEVENDEMON' => null,
                    'VREVENTOFOLHA' => null,
                    'NRORDEIMPRES' => null,
                    'ORDEMTIPOEVENTO' => null,
                    'NRVINCULOM' => null,
                    'NRCALCULOFOLHAVINC' => null
                );    
            }
            
            return $lista;
        }
    }    
    
    public function _executaProvisaoOtimizada(){
        
        $this->valorComplemento = 0;
        
        $this->valorComplemento = $this->_itens_calculo_principal->calculaIncidencias(FpatipoincideEntity::COMP_SAL_PROVISAO, $this->_mes_competencia, $this->_eventom);
        
        $this->calcula13SalProvisao();
        
        echo "calcula13SalProvisao".PHP_EOL;
        
        $this->calculaFerias();
        
        echo "calculaFerias".PHP_EOL;
        
        $this->calculaEncargos();
        
        echo "calculaEncargos".PHP_EOL;
        
        if($this->_mes_competencia->format('m/Y') == $this->_vinculo->getDtadmissaovinc()->format('m/Y')){
            $this->calculaDiferencaAvoAdmissao();
        }else{
            $this->calculaDiferenca();
        }
        
        echo "calculaDiferenca".PHP_EOL;
    }
    
    public function calculaDiferencaAvoAdmissao(){
        
        $gera_diferença = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, 'ID_GER_DIF_PROVISAO');
        
        if($gera_diferença == '1') {

            $obj_tp_calc_mov = current($this->_tpCalcMov->retornaPosicaoTpMov(FpatipomovimentEntity::TP_MOV_MENSAL, 0));
            $this->_itens_provisao_mes_anterior = $obj_tp_calc_mov->getInstancia();
            
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_PROV_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_DIFP_FERIAS_VENC, $this->_mes_competencia
            );
            
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_ENC1_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_DIF1_FERIAS_VENC, $this->_mes_competencia
            );
            
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_ENC2_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_DIF2_FERIAS_VENC, $this->_mes_competencia
            );
            
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_PROV_ADIC_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_DIFP_ADIC_FERIAS_VENC, $this->_mes_competencia
            );
            
            //Ferias Proporcional
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_PROV_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_DIFP_FERIAS_PROP, null
            );
            
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_ENC1_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_DIF1_FERIAS_PROP, null
            );
            
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_ENC2_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_DIF2_FERIAS_PROP, null
            );
            
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_PROV_ADIC_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_DIFP_ADIC_FERIAS_PROP, null
            );
            
            // 13
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_PROV_13SAL_CONT,
                FpatipoassoceveEntity::ASSOC_DIFP_13SAL_CONT, $this->_mes_competencia
            );
            
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_ENC1_13SAL_CONT,
                FpatipoassoceveEntity::ASSOC_DIF1_13SAL_CONT, $this->_mes_competencia
            );
            
            $this->_geraDiferencaAvo(
                $this->_itens_provisao_mes_anterior, FpatipoassoceveEntity::ASSOC_ENC2_13SAL_CONT,
                FpatipoassoceveEntity::ASSOC_DIF2_13SAL_CONT, $this->_mes_competencia
            );
        }
    }

    
    public function calculaDiferenca(){
        
        $gera_diferença = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, 'ID_GER_DIF_PROVISAO');
        
        $mesAnterior = DateUtil::subtraiIntervalo($this->_mes_competencia, 1, DateUtil::MES);
        
            
        if($gera_diferença == '1') {

            $calc_folhaAnterior = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                $this->_nrorg, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $mesAnterior, $mesAnterior
            )[0];
            
            $itemCalcCompetenciaAnterior = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAnterior->getNrcalculofolha());
            $itemCalcCompetenciaAnterior->retornaItensPorVinculo($this->_vinculo->getNrorg(), $this->_vinculo->getNrvinculom(), $calc_folhaAnterior->getNrcalculofolha());
            
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_PROV_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_DIFP_FERIAS_VENC, null
            );
            
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_ENC1_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_DIF1_FERIAS_VENC, $this->_mes_competencia
            );
            
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_ENC2_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_DIF2_FERIAS_VENC, $this->_mes_competencia
            );
            
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_PROV_ADIC_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_DIFP_ADIC_FERIAS_VENC, null
            );
            
            //Ferias Proporcional
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_PROV_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_DIFP_FERIAS_PROP, null
            );
            
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_ENC1_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_DIF1_FERIAS_PROP, $this->_mes_competencia
            );
            
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_ENC2_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_DIF2_FERIAS_PROP, $this->_mes_competencia
            );
            
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_PROV_ADIC_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_DIFP_ADIC_FERIAS_PROP, null
            );
            
            // 13
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_PROV_13SAL_CONT,
                FpatipoassoceveEntity::ASSOC_DIFP_13SAL_CONT, $this->_mes_competencia
            );
            
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_ENC1_13SAL_CONT,
                FpatipoassoceveEntity::ASSOC_DIF1_13SAL_CONT, $this->_mes_competencia
            );
            
            $this->_geraDiferencaMesAnterior(
                $itemCalcCompetenciaAnterior, FpatipoassoceveEntity::ASSOC_ENC2_13SAL_CONT,
                FpatipoassoceveEntity::ASSOC_DIF2_13SAL_CONT, $this->_mes_competencia
            );
        }
    }
    
    private function _geraDiferencaAvo(
        $item_calc_fonte, $assoc_fonte, $assoc_destino, $dt_ocorrencia_destino = null
    ){
        $evento_aux = $this->_assoc_eventos->retornaEventoAssociado($assoc_fonte, $this->_eventom, $this->_nrorg);
        $posicoes = $item_calc_fonte->retornaItens($evento_aux);
        $evento_aux = $this->_assoc_eventos->retornaEventoAssociado($assoc_destino, $this->_eventom, $this->_nrorg);
        
        foreach ($posicoes as $indice) {
            $referencia = $item_calc_fonte->retornaReferencia($indice);
            $valor_item = $item_calc_fonte->retornaValor($indice);
            if ($dt_ocorrencia_destino === null) {
                $dtocorrencia = $item_calc_fonte->retornaDtocorrencia($indice);
            } else {
                $dtocorrencia = $dt_ocorrencia_destino;
            }

            /*bloco para tratar valor de férias vencidas e encargos que deve sempre ser sobre 1/12*/
            if($assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_FERIAS_VENC
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIF1_FERIAS_VENC
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIF2_FERIAS_VENC
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_ADIC_FERIAS_VENC
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIF1_ADIC_FERIAS_VENC
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIF2_ADIC_FERIAS_VENC)
            {
                $valor_item=($valor_item/12);
            }
            /*bloco para tratar referencia apenas de férias vencidas que deve sempre ser 1/12*/
            if($assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_FERIAS_VENC
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_ADIC_FERIAS_VENC)
            {
                $referencia=2.5;
            }
            
            /*bloco para tratar valor de férias proporcionais e encargos que deve sempre ser sobre 1/12*/
            if(   $assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_FERIAS_PROP
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIF1_FERIAS_PROP
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIF2_FERIAS_PROP
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_ADIC_FERIAS_PROP
                ||$assoc_destino== FpatipoassoceveEntity::ASSOC_DIF1_ADIC_FERIAS_PROP
                ||$assoc_destino== FpatipoassoceveEntity::ASSOC_DIF2_ADIC_FERIAS_PROP)
            {
                $evento_auxfer = $this->_assoc_eventos->retornaEventoAssociado(FpatipoassoceveEntity::ASSOC_PROV_FERIAS_PROP, $this->_eventom, $this->_nrorg);
                $posicao = $item_calc_fonte->retornaItens($evento_auxfer);
                foreach ($posicao as $ind) {
                    $avoFerProp = $item_calc_fonte->retornaReferencia($ind);
                }
                
                $valor_item=($valor_item/$avoFerProp);
            }
            /*bloco para tratar referencia apenas de férias proporcionais que deve sempre ser 1/12*/
            if($assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_FERIAS_PROP
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_ADIC_FERIAS_PROP)
            {
                  $referencia=1;
            }
            /*bloco para tratar valor de 13º e encargos que deve sempre ser sobre 1/12*/
            if(   $assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_13SAL_CONT
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIF1_13SAL_CONT
                ||$assoc_destino==FpatipoassoceveEntity::ASSOC_DIF2_13SAL_CONT
            )
            {
                $evento_aux13 = $this->_assoc_eventos->retornaEventoAssociado(FpatipoassoceveEntity::ASSOC_PROV_13SAL_CONT, $this->_eventom, $this->_nrorg);
                $posicao = $item_calc_fonte->retornaItens($evento_aux13);
                
                foreach ($posicao as $ind) {
                    $avo13 = $item_calc_fonte->retornaReferencia($ind);
                }
                $valor_item=($valor_item/$avo13);
            }
            
            /*bloco para tratar referencia apenas de 13º que deve sempre ser 1/12*/
            if($assoc_destino==FpatipoassoceveEntity::ASSOC_DIFP_13SAL_CONT)
            {
               $referencia=1;
            }
            
            if( ($referencia!=0 || $valor_item!=0) && $valor_item < 9999999){
                $this->_itens_calculo_principal->geraEvento($evento_aux, $dtocorrencia, $valor_item, $referencia, 1);
            }
        }
    }
    
    public function calculaEncargos(){
        
        $vinculo = $this->_vinculo;
        
        if (in_array(FpaProcessoEntity::FGTS, $this->_listaProcessos) === true) {

            if ($vinculo->getNrvinculoempreg() == GpevinculohEntity::VINCULO_EMPREGATICIO_MENOR_APRENDIZ) {
                $percentual = GpevinculohEntity::PERCENTUAL_FGTS_MENOR_APRENDIZ;
            } else{
                $percentual = $this->_valores_referencia->getValorReferencia(FpaValorreferenciamEntity::FGTS);
            }
            FGTS::calcula(
                FpatipoincideEntity::INC_ENC_2_PROV_FER_VENC, FpatipoassoceveEntity::ASSOC_ENC2_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_BASE_ENC2_PROV_FER_VENC, $this->_mes_competencia, $this->_itens_calculo_principal,
                $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
            );
        }
        
        if (in_array(FpaProcessoEntity::INSS, $this->_listaProcessos) === true) {
            INSS::calcula(
                FpatipoincideEntity::INC_ENC_1_PROV_FER_VENC, FpatipoassoceveEntity::ASSOC_ENC1_FERIAS_VENC,
                FpatipoassoceveEntity::ASSOC_BASE_ENC1_PROV_FER_VENC, $this->_mes_competencia, $this->_itens_calculo_principal,
                $this->_assoc_eventos, $this->_eventom, $this->_nrorg, $this->_vrEnc1Ferias, null, null, 0, false
            );
        }
        /*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=FERIAS PROPORCIONAIS=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
        
        if (in_array(FpaProcessoEntity::FGTS, $this->_listaProcessos) === true) {

            if ($vinculo->getNrvinculoempreg() == GpevinculohEntity::VINCULO_EMPREGATICIO_MENOR_APRENDIZ) {
                $percentual = GpevinculohEntity::PERCENTUAL_FGTS_MENOR_APRENDIZ;
            } else{
                $percentual = $this->_valores_referencia->getValorReferencia(FpaValorreferenciamEntity::FGTS);
            }
            
            FGTS::calcula(
                FpatipoincideEntity::INC_ENC_2_PROV_FER_PROP, FpatipoassoceveEntity::ASSOC_ENC2_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_BASE_ENC2_PROV_FER_PROP, $this->_mes_competencia, $this->_itens_calculo_principal,
                $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
            );
        }
        
        if (in_array(FpaProcessoEntity::INSS, $this->_listaProcessos) === true) {
            INSS::calcula(
                FpatipoincideEntity::INC_ENC_1_PROV_FER_PROP, FpatipoassoceveEntity::ASSOC_ENC1_FERIAS_PROP,
                FpatipoassoceveEntity::ASSOC_BASE_ENC1_PROV_FER_PROP, $this->_mes_competencia, $this->_itens_calculo_principal,
                $this->_assoc_eventos, $this->_eventom, $this->_nrorg, $this->_vrEnc1Ferias, null, null, 0, false
            );
        }

        /*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-DECIMO TERCEIRO=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
        
        if (in_array(FpaProcessoEntity::FGTS, $this->_listaProcessos) === true) {

            if ($vinculo->getNrvinculoempreg() == GpevinculohEntity::VINCULO_EMPREGATICIO_MENOR_APRENDIZ) {
                $percentual = GpevinculohEntity::PERCENTUAL_FGTS_MENOR_APRENDIZ;
            } else{
                $percentual = $this->_valores_referencia->getValorReferencia(FpaValorreferenciamEntity::FGTS);
            }
            FGTS::calcula(
                FpatipoincideEntity::INC_ENC_2_PROV_DEC_TERC, FpatipoassoceveEntity::ASSOC_ENC2_13SAL_CONT,
                FpatipoassoceveEntity::ASSOC_BASE_ENC2_PROV_DEC_TERC, $this->_mes_competencia, $this->_itens_calculo_principal,
                $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
            );
        }
        
        if (in_array(FpaProcessoEntity::INSS, $this->_listaProcessos) === true) {
            INSS::calcula(
                FpatipoincideEntity::INC_ENC_1_PROV_DEC_TERC, FpatipoassoceveEntity::ASSOC_ENC1_13SAL_CONT,
                FpatipoassoceveEntity::ASSOC_BASE_ENC1_PROV_DEC_TERC, $this->_mes_competencia, $this->_itens_calculo_principal,
                $this->_assoc_eventos, $this->_eventom, $this->_nrorg, $this->_vrEnc1DecTerceiro, null, null, 0, false
            );
        }
    }
    
    
    public function calculaFerias(){
        
        $vinculo = $this->_vinculo;
        $dataRescisaoProjetado = DateUtil::getDataDeString($this->_ult_dia_mes);
        
        $ferias = $this->_ferias->verificaFeriasProvisao($vinculo, $dataRescisaoProjetado, true, $this->_eventom,
            $this->_faltas_intervalo, $this->_jornada_diaria, $this->_nrTipoModalidadeCalc, $this->_nrOcorrenciaCalc,
            $this->_alteracao_situacao_funcional, $this->_mes_competencia, $this->_jornada_semanal);
        
        $contador_ferias_vencidas = 0;
        
        for ($i=0; $i < count($ferias); $i++) {

            $dtInicioAquisicao = DateUtil::getDataDeString($ferias[$i]['DTINIAQUISICAO']);
            $dtFimAquisicao = DateUtil::getDataDeString($ferias[$i]['DTFIMAQUISICAO']);
           
            $multperda= FpatipoferiasmEntity::retornaDiasDireito($ferias[$i]['TIPOFERIAS']);
            $proporcaoPerda=1/($multperda*12)*$ferias[$i]['QTDIASMAXFE'];
            
            if($dtFimAquisicao >= $this->_mes_competencia){
                $ferias[$i]['IDCONTROLFERIAS'] = FpaferiasEntity::IDCONTROLFERIAS_ABERTO;
            }
            
            switch ($ferias[$i]['IDCONTROLFERIAS']){
                case FpaferiasEntity::IDCONTROLFERIAS_ABERTO:
                    
                    $listaNrEventos = $this->_eventomMedia->retornaListaEventoMedia();

                    $dtInicioAquisicao = DateUtil::setDay($dtInicioAquisicao, 1);
                    $dataAuxiliar = $dtFimAquisicao;
                    
                    $inicio_periodo = $dtInicioAquisicao;
                    $fim_periodo = $dtFimAquisicao;
                    
                    $idMediaUltMes = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, 'ID_MEDIA_ULT_MES');
                    
                    if($idMediaUltMes == 1) {
                        $nrSindicato = $vinculo->historico->getNrestrutsind();
                        if ($nrSindicato) {
                            $fpaparamsindicato = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->findOneBy(array('nrestruturam' => $nrSindicato, 'nrorg' => $vinculo->getNrorg()));
                            if ($fpaparamsindicato && $fpaparamsindicato->getNrmesesmedia1() < 12) {
                                $vrParamSindicato = $fpaparamsindicato->getNrmesesmedia1();
                                $fim_periodo = $this->_mes_competencia;
                                
                                if($this->_iddesccompetenc == 'S'){
                                    $fim_periodo = DateUtil::subtraiIntervalo($fim_periodo, 1);
                                }else{
                                    $fim_periodo = DateUtil::getUltimoDiaDoMes($fim_periodo);
                                }
                        
                                $inicio_periodo = DateUtil::subtraiIntervalo($fim_periodo, $vrParamSindicato - 1, DateUtil::MES);
                                $inicio_periodo = DateUtil::setDay($inicio_periodo, 1);
                            }
                        }
                    }
                    
                    $listaEventosTotalizados = $this->_media->calculaMaiorMedia(
                        $vinculo, FpatipomovimentEntity::TP_MOV_MENSAL, $listaNrEventos, $inicio_periodo, $fim_periodo,
                        1, null
                    );

                    $listaEventosOrigemDestino = array_combine(preg_split('/[\s,]+/', $listaNrEventos), preg_split('/[\s,]+/', $listaNrEventos));

                    $this->_divisor_ferias =  $this->_calculaDivisorMedias($inicio_periodo, $fim_periodo);

                    if($this->_divisor_fixo != false ){
                        $this->_divisor_ferias = $this->_divisor_fixo;
                    }
                    
                    $vr_media = $this->_media->retornaEventoMedia(
                        $this->_itens_calculo_principal, $listaEventosTotalizados, $listaEventosOrigemDestino,
                        FpatipomovimentEntity::TP_MOV_FERIAS, true, $this->_divisor_ferias, null, $this->_salario, $this->_jornada_diaria
                    );
                    
                    /*$baseFerProp = $this->_calculaRemuneracaoBase($vinculo, $this->_itens_ferias_proporcionais_rescisao, FpatipocalculoEntity::CALC_FERIAS,
                        FpatipoincideEntity::INC_FERIAS
                    );*/
                    
                    $baseFerProp = $this->_salario + $this->valorComplemento + $vr_media; 
                    
                    $val = ($baseFerProp / 12) * ($ferias[$i]['QTMESESMAXFE'] * $proporcaoPerda);
                    
                    // Ferias Proporcional
                    $evento_FeriasProporcional = $this->_assoc_eventos->retornaEventoAssociado(
                        FpatipoassoceveEntity::ASSOC_PROV_FERIAS_PROP, $this->_eventom, $this->_nrorg
                    );
                    $this->_itens_calculo_principal->geraEvento($evento_FeriasProporcional, $dtInicioAquisicao, $val, $ferias[$i]['QTMESESMAXFE'] * $proporcaoPerda, 1);
                    
                    // 1/3 Proporcional
                    $evento_FeriasProporcionalAdic = $this->_assoc_eventos->retornaEventoAssociado(
                        FpatipoassoceveEntity::ASSOC_PROV_ADIC_FERIAS_PROP, $this->_eventom, $this->_nrorg
                    );
                    $this->_itens_calculo_principal->geraEvento($evento_FeriasProporcionalAdic, $dtInicioAquisicao, ($val / 100) * 33.3333333333333, $ferias[$i]['QTMESESMAXFE'] * $proporcaoPerda, 1);
                    
                    // Media 
                    $evento_media = $this->_assoc_eventos->retornaEventoAssociado(
                        FpatipoassoceveEntity::ASSOC_PROV_MEDIAS_FERIAS_PROP, $this->_eventom, $this->_nrorg
                    );
                    $this->_itens_calculo_principal->geraEvento($evento_media, $dtInicioAquisicao, $vr_media, 0, 1);
            
                    break;

                case FpaferiasEntity::IDCONTROLFERIAS_ADQUIRIDO:
                case FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL:
                                       
                        $diasGozados = $ferias[$i]['QTDDIASGOZADOS'];

                        //$this->verificaDobraFeriasRescisaoProvisao($vinculo,$ferias[$i],$diasGozados,$contador_ferias_vencidas);
                        
                        $listaNrEventos = $this->_eventomMedia->retornaListaEventoMedia();
                        if($dtInicioAquisicao->format('d') != 1){
                            $dtFimAquisicao = DateUtil::getUltimoDiaDoMes(DateUtil::subtraiIntervalo($dtFimAquisicao, 1, DateUtil::MES));
                        }
                        $dtInicioAquisicao = DateUtil::setDay($dtInicioAquisicao, 1);
                        
                        $inicio_periodo = $dtInicioAquisicao;
                        $fim_periodo = $dtFimAquisicao;
                        
                        $idMediaUltMes = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, 'ID_MEDIA_ULT_MES');
                        
                        if($idMediaUltMes == 1) {
                            $nrSindicato = $vinculo->historico->getNrestrutsind();
                            if ($nrSindicato) {
                                $fpaparamsindicato = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->findOneBy(array('nrestruturam' => $nrSindicato, 'nrorg' => $vinculo->getNrorg()));
                                if ($fpaparamsindicato && $fpaparamsindicato->getNrmesesmedia1() < 12) {
                                    $vrParamSindicato = $fpaparamsindicato->getNrmesesmedia1();
                                    $fim_periodo = $this->_mes_competencia;
                                    
                                    if($this->_iddesccompetenc == 'S'){
                                        $fim_periodo = DateUtil::subtraiIntervalo($fim_periodo, 1);
                                    }else{
                                        $fim_periodo = DateUtil::getUltimoDiaDoMes($fim_periodo);
                                    }
                            
                                    $inicio_periodo = DateUtil::subtraiIntervalo($fim_periodo, $vrParamSindicato - 1, DateUtil::MES);
                                    $inicio_periodo = DateUtil::setDay($inicio_periodo, 1);
                                }
                            }
                        }
                        
                        $listaEventosTotalizados = $this->_media->calculaMaiorMedia(
                            $vinculo, FpatipomovimentEntity::TP_MOV_MENSAL, $listaNrEventos, $inicio_periodo, $fim_periodo,
                            1, null
                        );
                        $listaEventosOrigemDestino = array_combine(preg_split('/[\s,]+/', $listaNrEventos), preg_split('/[\s,]+/', $listaNrEventos));
                        
                        $this->_divisor_ferias =  $this->_calculaDivisorMedias($inicio_periodo, $fim_periodo);
                        
                        $vr_media = $this->_media->retornaEventoMedia(
                            $this->_itens_calculo_principal, $listaEventosTotalizados,
                            $listaEventosOrigemDestino, FpatipomovimentEntity::TP_MOV_FERIAS, true, $this->_divisor_ferias, null, $this->_salario, $this->_jornada_diaria
                        );
                        
                        /*$base = $this->_calculaRemuneracaoBase(
                           $vinculo, $this->_itens_ferias_vencidas_rescisao, FpatipocalculoEntity::CALC_FERIAS,
                            FpatipoincideEntity::INC_FERIAS
                        );*/
                        
                        $base = $this->_salario + $this->valorComplemento + $vr_media; 
                    
                        $val = ($base / 30) * ($ferias[$i]['QTDIASMAXFE'] - $diasGozados);
                        
                        // Ferias Vencidas
                        $evento_FeriasVencidas = $this->_assoc_eventos->retornaEventoAssociado(
                            FpatipoassoceveEntity::ASSOC_PROV_FERIAS_VENC, $this->_eventom, $this->_nrorg
                        );
                        $this->_itens_calculo_principal->geraEvento($evento_FeriasVencidas, $dtInicioAquisicao, $val, $ferias[$i]['QTDIASMAXFE'] - $diasGozados, 1);
            
                        // Ferias Vencidas 1/3
                        $evento_FeriasVencidasAdic = $this->_assoc_eventos->retornaEventoAssociado(
                            FpatipoassoceveEntity::ASSOC_PROV_ADIC_FERIAS_VENC, $this->_eventom, $this->_nrorg
                        );
                        $this->_itens_calculo_principal->geraEvento($evento_FeriasVencidasAdic, $dtInicioAquisicao, ($val / 100) * 33.3333333333333, $ferias[$i]['QTDIASMAXFE'] - $diasGozados, 1);
                        
                        // Media Ferias vencidas
                        $evento_media = $this->_assoc_eventos->retornaEventoAssociado(
                            FpatipoassoceveEntity::ASSOC_PROV_MEDIAS_FERIAS_VENC, $this->_eventom, $this->_nrorg
                        );
                        $this->_itens_calculo_principal->geraEvento($evento_media, $dtInicioAquisicao, $vr_media, 0, 1);
                        
                    //$contador_ferias_vencidas++;
                    break;
            }
            
            unset($dtInicioAquisicao);
            unset($dtFimAquisicao);
        }
    }
    
    public function calcula13SalProvisao(){
        
        $dt_admissao = DateUtil::getDataDeString($this->_vinculo->getDtadmissaovinc());
        $dt_rescisao = $this->_vinculo->getDtrescisaovinc();

        if (empty($dt_rescisao) === false) {
            $dt_rescisao = DateUtil::getDataDeString($this->_vinculo->getDtrescisaovinc());
        } else {
            $dt_rescisao = null;
        }

        $this->_atribuiInicioPeriodo13Salario($dt_admissao);
        $this->_fim_periodo_13sal = $this->_ult_dia_mes;

        $this->_alteracao_situacao_funcional->retornaAlteracaoSituacaoFuncional(
            $this->_vinculo->getNrorg(), $this->_vinculo->getNrvinculom(), $this->_vinculo->getDtadmissaovinc()
        );
        
        $fpaparamsindicato = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->findOneBy(
            array('nrorg' => $this->_vinculo->getNrorg(), 'nrestruturam' => $this->_vinculo->historico->getNrestrutsind()));
            
        if(is_object($fpaparamsindicato)) {
            $diasminimo = $fpaparamsindicato->getNrdiv13antecip();
        } else {
            $diasminimo = 15;
        }

        $dtfinal = DateUtil::getUltimoDiaDoMes(DateUtil::getDataAtual(true));

        $this->_divisor_13 = $this->_calculaNrDozeAvosDecimoTerceiro($this->_vinculo,
            $this->_inicio_periodo_13sal, $dtfinal, $diasminimo);
            
        $this->_nr_doze_avos = $this->_calculaNrDozeAvosDecimoTerceiro($this->_vinculo,
            $this->_inicio_periodo_13sal, $this->_fim_periodo_13sal
        );
        
        if ($this->_nr_doze_avos > 0) {
            
            $this->_inicio_periodo_13sal = DateUtil::setDay($this->_inicio_periodo_13sal, 1);
            
            $listaNrEventos = $this->_eventomMedia->retornaListaEventoMedia();
            if (((int)$this->_fim_periodo_13sal->format('m') == 12) and ($this->_nrOcorrenciaCalculo > 1)) {
                $dataAuxiliar = $this->_fim_periodo_13sal;
            }else {
                $dataAuxiliar = $this->_fim_periodo_13sal;
            }
            
            $inicio_periodo = $this->_inicio_periodo_13sal;
            $fim_periodo = $dataAuxiliar;
            
            $idMediaUltMes = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, 'ID_MEDIA_ULT_MES');
            
            if($idMediaUltMes == 1) {
                $nrSindicato = $this->_vinculo->historico->getNrestrutsind();
                if ($nrSindicato) {
                    $fpaparamsindicato = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->findOneBy(array('nrestruturam' => $nrSindicato, 'nrorg' => $this->_vinculo->getNrorg()));
                    if ($fpaparamsindicato && $fpaparamsindicato->getNrmesesmedia1() < 12) {
                        $vrParamSindicato = $fpaparamsindicato->getNrmesesmedia1();
                        $fim_periodo = $this->_mes_competencia;
                        
                        if($this->_iddesccompetenc == 'S'){
                            $fim_periodo = DateUtil::subtraiIntervalo($fim_periodo, 1);
                        }else{
                            $fim_periodo = DateUtil::getUltimoDiaDoMes($fim_periodo);
                        }
                
                        $inicio_periodo = DateUtil::subtraiIntervalo($fim_periodo, $vrParamSindicato - 1, DateUtil::MES);
                        $inicio_periodo = DateUtil::setDay($inicio_periodo, 1);
                    }
                }
            }
            
            $listaEventosTotalizados = $this->_media->calculaMaiorMedia(
                $this->_vinculo, FpatipomovimentEntity::TP_MOV_MENSAL, $listaNrEventos, $inicio_periodo,
                $fim_periodo, 1, null
            );
            $listaEventosOrigemDestino = array_combine(preg_split('/[\s,]+/', $listaNrEventos), preg_split('/[\s,]+/', $listaNrEventos));
            
            $this->_divisor_13 =  $this->_calculaDivisorMedias($inicio_periodo, $fim_periodo);
            if($this->_divisor_fixo){
                $this->_divisor_13 = $this->_divisor_fixo;
            }
            
            $vr_media = $this->_media->retornaEventoMedia(
                $this->_itens_calculo_principal, $listaEventosTotalizados, $listaEventosOrigemDestino, FpatipomovimentEntity::TP_MOV_13SAL,
                true, $this->_divisor_13, null, $this->_salario, $this->_jornada_diaria
            );
            
            /*$base = $this->_calculaRemuneracaoBase(
               $vinculo, $this->_itens_13sal_resc, FpatipocalculoEntity::CALC_13SAL, FpatipoincideEntity::INC_13SAL
            );*/
            
            $base = $this->_salario + $this->valorComplemento + $vr_media; 
            
            $evento_media = $this->_assoc_eventos->retornaEventoAssociado(
                FpatipoassoceveEntity::ASSOC_PROV_MEDIAS_13SAL, $this->_eventom, $this->_nrorg
            );
            $this->_itens_calculo_principal->geraEvento($evento_media, null, $vr_media, $this->_nr_doze_avos, 1);
            
            $val = ($base / 12) * $this->_nr_doze_avos;
            
            $evento_13 = $this->_assoc_eventos->retornaEventoAssociado(
                FpatipoassoceveEntity::ASSOC_PROV_13SAL_CONT, $this->_eventom, $this->_nrorg
            );
            
            $this->_itens_calculo_principal->geraEvento($evento_13, null, $val, $this->_nr_doze_avos, 1);
            
        }
    }
    
    private function _atribuiInicioPeriodo13Salario($dtAdmissao){
        if ($dtAdmissao > $this->_primeiroDiaAno) {
            $this->_inicio_periodo_13sal = $dtAdmissao;
        } else {
            $this->_inicio_periodo_13sal = $this->_primeiroDiaAno;
        }
    }
    
    private function _calculaNrDozeAvosDecimoTerceiro($vinculo, $dt_inicio, $dt_fim, $qtMinimoDias=15){
        
        $dt_inicio = DateUtil::getDataDeString($dt_inicio);
        $dt_fim = DateUtil::getDataDeString($dt_fim);

        $mes_ini = $dt_inicio->format('m');
        $mes_fim = $dt_fim->format('m');
        $nr_doze_avos = 0;
        $dt_aux_inicio = $dt_inicio;
        
        $ano_ini=$dt_inicio->format('Y');
        $ano_fim=$dt_fim->format('Y');
        if($ano_fim>$ano_ini){
            $mes_fim=13;
        }
        
        for ($i = $mes_ini; $i<=($mes_fim); $i++) {
            if ($i == $mes_ini) {
                $dt_aux_inicio = $dt_inicio;
                $dt_aux_fim = DateUtil::getUltimoDiaDoMes($dt_inicio);
                if($dt_aux_fim > $dt_fim){                          
                    $dt_aux_fim = $dt_fim;      
                }
            } 
            elseif ($i == $mes_fim) {
                $dt_aux_inicio = DateUtil::setDay($dt_fim, 1);
                $dt_aux_fim = $dt_fim;
            } else {
                $dt_aux_inicio =  DateUtil::setDay(DateUtil::setMonth($dt_inicio, $i), 1);
                $dt_aux_fim = DateUtil::getUltimoDiaDoMes($dt_aux_inicio);
            }
            $dias_intervalo = DateUtil::getQtdeDiasEntre($dt_aux_inicio, $dt_aux_fim) + 1;
            
            if ($dias_intervalo >= $qtMinimoDias) {
                $dias_afastados = $this->_quantidadeDiasAfastados($dt_aux_inicio, $dt_aux_fim);
                $dias_faltas= $this->_quantidadeFaltas($vinculo, $dt_aux_inicio, $dt_aux_fim);
                
                if (($dias_intervalo - $dias_afastados - $dias_faltas) >= $qtMinimoDias) {
                    $nr_doze_avos++;
                }
            }
        }
        
        return $nr_doze_avos;

    }
    
    private function _calculaDivisorMedias($dt_inicio,$dt_fim){

        $data1 = $dt_inicio;
        $arr = explode('/',$data1);

        $data2 = $dt_fim;
        $arr2 = explode('/',$data2);

        $mes1 = $arr[1];
        $ano1 = $arr[2];

        $mes2 = $arr2[1];
        $ano2 = $arr2[2];

        $a1 = ($ano2 - $ano1)*12;
        $m1 = ($mes2 - $mes1)+1;
        $divisor = ($m1 + $a1);

        if ($divisor>12){
            $divisor=12;
        }

        return $divisor;

    }
    
}