<?php
namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use Doctrine\ORM\EntityManager;
use HCM\FPA\Services\LoteLancEventoService;
use HCM\FPA\Services\ExportacaoExcelService;
use HCM\FPA\Services\MigracaoFichaFinanceiraService;

class MigracaoFichaFinanceiraController extends Crud {

    protected $dataSourceGetLoteLancEvento = 'getLoteFichaFinanceira';
    protected $dataSourceSetLoteLancEvento = 'fpaLoteLancEvento_setLoteLancEvento';
    protected $dataSourceSetEventoDePara = 'migramdepara';


    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, ExportacaoExcelService $ExportacaoExcelService, MigracaoFichaFinanceiraService $migracaoFichaFinanceiraService, LoteLancEventoService $LoteLancEventoService){
        $this->parameterBag    = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager; 
        $this->ExportacaoExcelService = $ExportacaoExcelService;
        $this->migracaoFichaFinanceiraService = $migracaoFichaFinanceiraService; 
        $this->LoteLancEventoService = $LoteLancEventoService; 
        parent::__construct($dataSourceManager);
    }

    public function getLoteFichaFinanceira (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetLoteLancEvento);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setEventoDePara(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            
            $evento = $this->entityManager->getRepository(Repositories::MIGRAMDEPARA)->findBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrde' => $row['NRDE']));
            if(empty($evento)){
                $this->dataSourceName = $this->dataSourceSetEventoDePara;
                if (array_key_exists('NRMIGRAMDEPARA', $row)) {
                    if ($row['NRMIGRAMDEPARA'] === '') {
                        $row['NRMIGRAMDEPARA'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'MIGRAMDEPARA', 1, 12, true);
                    }
                } else {
                    $row['NRMIGRAMDEPARA'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'MIGRAMDEPARA', 1, 12, true);
                }
                $row['NMTABELADBF'] = isset($row['NMTABELADBF'])?$row['NMTABELADBF']:"EVENTO";
                
                parent::save($request, $response);
            }else{
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Não foi possível inserir, já existe registro para o Nr evento de: '.$row['NRDE'].'', DTO\Response\Message::TYPE_MESSAGE));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setLoteFichaFinanceira(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceSetLoteLancEvento;
        try{
            $msg = array();
            $disponibilidade = true;
            foreach($request->getDataSet()->getRows() as $row) {
                if (isset($row['NRTIPOVINCULOM'])) {
                    $bloq = $this->LoteLancEventoService->verificaEventoBloqueado($row['NRORG'], $row['NREVENTOM'], $row['NRTIPOVINCULOM']);
                    if ($bloq !== null){
                        if ($bloq['NREVENTOMPARA'] === null) {
                            $this->msg[] = array('msg' => 'Operação bloqueada. O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM']. '"!');
                            $disponibilidade = false;
                        } else {
                            $msg[] = array('msg' => 'Operação bloqueada.  O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM'].'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                            $disponibilidade = true;
                            $row['NREVENTOM'] = $bloq['NREVENTOMPARA'];
                        }
                    } else {
                        $disponibilidade = true;
                    }
                }
                if ($disponibilidade === true) {
                    if($row['__is_new'] === true){
                        $row['NRLOTELANCEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_LOTELANCEVENTO', 1, 12, true);
                    } 
                    if(!isset($row['IDORIGLOTELANC'])){
                        $row['IDORIGLOTELANC'] = 'MIGRACAO';
                    }
                    if(isset($row['EXCLUIITENSZERADOS'])){ 
                        $this->LoteLancEventoService->deteleItensLoteZerados($row);
                    }
                    
                    parent::save($request, $response);
                }
            }
            if (count($msg) > 0) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function baixarPlanilhaTemplateEventosDePara(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            
            $file = $this->migracaoFichaFinanceiraService->baixarPlanilhaTemplateEventosDePara($row);

            if($file !== null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function baixarPlanilhaTemplateLoteFicha(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            
            $file = $this->migracaoFichaFinanceiraService->baixarPlanilhaTemplateLoteFicha($row);

            if($file !== null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarPlanilhaEventoDePara(DTO\Request\DataSet $request, DTO\Response $response){
        set_time_limit(0);
        try{
            $dataset = $request->getDataSet()->getRows();
            
            $row = $dataset[0];
            
            $retorno = $this->migracaoFichaFinanceiraService->importarPlanilhaEventoDePara($row);
            $mensagens = !empty($retorno['messages']) ? $this->migracaoFichaFinanceiraService->preparaMensagens($retorno['messages']) : null;
            
            if($retorno['success'] == true){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
            }
            
            if(!empty($mensagens)){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $mensagens));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarPlanilhaLoteFicha(DTO\Request\DataSet $request, DTO\Response $response){
        set_time_limit(0);
        try{
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            
            $retorno = $this->migracaoFichaFinanceiraService->importarPlanilhaLoteFicha($row);
            
            if($retorno['success'] == true){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($retorno['messages'][0], \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $mensagens = array_key_exists('messages', $retorno) && !empty($retorno['messages']) ? $this->migracaoFichaFinanceiraService->preparaMensagens($retorno['messages']) : null;
            
                if(!empty($mensagens)){
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $mensagens));
                }
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function exportarLoteFicha(DTO\Request\DataSet $request, DTO\Response $response){
        set_time_limit(0);
        try{
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            
            $retorno = $this->migracaoFichaFinanceiraService->exportarLoteFicha($row);
            
            if($retorno['success'] == true){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($retorno['messages'][0], \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $mensagens = array_key_exists('messages', $retorno) && !empty($retorno['messages']) ? $this->migracaoFichaFinanceiraService->preparaMensagens($retorno['messages']) : null;
            
                if(!empty($mensagens)){
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $mensagens));
                }
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na exportação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function cancelarLoteFichaFinanceira(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            
            $mensagens = $this->migracaoFichaFinanceiraService->cancelarLoteFichaFinanceira($row);
            
            if(is_array($mensagens)){
                $retornoMensagens = $this->migracaoFichaFinanceiraService->preparaMensagens($mensagens);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('mensagens', $retornoMensagens));    
            }else if($mensagens == true){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Lote cancelado com sucesso', DTO\Response\Message::TYPE_MESSAGE));
            }else{
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Lote não foi cancelado.', DTO\Response\Message::TYPE_MESSAGE));
            }    
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
    
}