<?php
namespace HCM\Util\ModeloImportacao;

use HCM\Util\ModeloImportacao\References\Autoincrement;
use HCM\Util\ModeloImportacao\References\Executavel;
use HCM\Util\ModeloImportacao\References\FixedValue;
use HCM\Util\ModeloImportacao\References\ForeignKey;
use HCM\Util\ModeloImportacao\References\Label;
use HCM\Util\ModeloImportacao\References\PadraoInvalido;
use HCM\Util\ModeloImportacao\References\Reference;
use HCM\Util\ModeloImportacao\References\ValorCampo;
use HCM\Util\ModeloImportacao\References\FixedField;

/**
 * Constroi o conjunto de dados que sera utilizado para a criacao dos arquivos XML para transmissao.
 *
 * Class DataBuilder
 * @package HCM\Util\ModeloImportacao
 */
class DataBuilderESocial
{
	private $estrutura;
	private $data;
	private $eventos;
	private $rules;
	private $fixedFields;
	private $padroesMetadados;
	private $entidades;
    private $excludedRegist;

    static $pk = array();

    /**
     * @param $estrutura    array Relacao entre os dados do banco de dados e os templates twig (esturura.json).
     * @param $data         array Array com todos os dados retornados do banco de dados.
     * @param $helpers      array Array com todos os dados auxiliares dos eventos (helper.json).
     */
    public function __construct($estrutura, $data, $helpers)
	{
		$this->estrutura       = $estrutura;
		$this->data            = $data;
		$this->rules           = $helpers["rules"];
        $this->fixedFields     = $helpers["fixedFields"];
        // Armazena o nome dos agrupadores (presentes no arquivo estrutura.json) que serão utilizados em determinado projeto de importacao.
        $this->eventos     = array();
        // Objeto onde todas as entidades criadas serao planificadas. Este array sera retornado no final do processo de geracao dos dados.
        // É publicamente acessivel pelo metodo addAgrupador.
        $this->entidades       = array();

        $this->excludedRegist = $helpers["excludedTables"];

        // Este array armazena todos os padroes que sao compreendidos pelo modelo, e sao utilizados nos arquivos de estrutura.json e helper.json. Qualquer classe
        // utilizada neste array deverá implementar a interface de HCM\Util\ModeloImportacao\References\Reference.
		$this->padroesMetadados = array(
			new ValorCampo(),
			new ForeignKey(),
            new Label(),
            new FixedField(),
            new FixedValue(),
            new Executavel(),
			new PadraoInvalido(),
		);
	}

	public function addEvento($evento)
	{
		$this->eventos[] = $evento;
	}

	public function build() 
	{
		$this->percorreLinhasEvento();
		// Retorna todos os dados planos e insere todos os campos padrao.
		return $this->entidades;
	}

    /**
     * Percorre todas as linhas de dados.
     */
    protected function percorreLinhasEvento()
	{
        foreach ($this->data as $key => $linhaDados) {
            $this->percorreEventos($linhaDados);
            unset($this->data[$key]);
		}
	}

    /**
     * Percorre todos os agrupamentos do arquivo estrutura.json
     *
     * @param $linhaDados array objeto contendo a linha corrente. Passa por este metodo somente para ser transmitido para os proximos,
     * onde este realmente sera necessario.
     */
    protected function percorreEventos($linhaDados)
	{
		foreach ($this->estrutura as $nomeEvento => $agrupadores) {
            if (in_array($nomeEvento, $this->eventos)) {
                $entityContainer = new EntityContainerESocial();
                $entityContainer->adicionaEvento($nomeEvento, new BasicEntityESocial($nomeEvento));
                $this->percorreAgrupadores($linhaDados, $nomeEvento, $agrupadores, $entityContainer);
//                if (isset($this->rules[$nomeAgrupamento])) {
//                    $this->executaRegrasDeTabelas($this->rules[$nomeAgrupamento], $entityContainer, $linhaDados);
//                }
                $this->setEntidades($entityContainer);
            }
		}
	}


    /**
     * Percorre todas os registros de determinado agrupamento do arquivo de estruturas (estrutura.json).
     *
     * @param $linhaDados     array             Linha de dados. Nao e utilizado por este metodo, somente transmitido para os
     *                                          metodos posteriores.
     * @param $registros      array             Array com todas as registros do agrupamento corrente.
     * @param EntityContainerESocial $entityContainer  Container que devera armazenar todas as entidades geradas a partir da linha.
     */
    protected function percorreAgrupadores($linhaDados, $nomeEvento, $agrupadores, EntityContainerESocial $entityContainer) {
        foreach ($agrupadores as $nomeAgrupador => $dadosAgrupador) {
            $this->percorreDados($linhaDados, $nomeEvento, $nomeAgrupador, $dadosAgrupador, $entityContainer);
            $this->addFixedFields($entityContainer->getEvento($nomeEvento));
		}
	}

    /**
     * Percorre todos os dados de um registro.
     *
     * @param $linhaDados     array            Linha do excel.
     * @param $nomeTabela     string           Nome da tabela.
     * @param $dadosTabela    array            conteudo das tabelas descritas em estrutura.json. Nao e utilizado aqui, mas
     *                                         transmitido para os metodos posteriores.
     * @param EntityContainerESocial $entityContainer Container que devera armazenar todas as entidades geradas a partir da linha.
     */
    protected function percorreDados($linhaDados, $nomeEvento, $nomeAgrupador, $dadosAgrupador, EntityContainerESocial $entityContainer)
	{
        foreach ($linhaDados as $nomeColunaDados => $valorColunaDados) {
            if (is_array($valorColunaDados)) {
                $valoresColunaDados = array();
                foreach ($valorColunaDados as $key => $linhaDadosColuna) {
                    $entityContainerColuna = new EntityContainerESocial();
                    $entityContainerColuna->adicionaEvento($nomeColunaDados, new BasicEntityESocial($nomeColunaDados));
                    $this->percorreDados($linhaDadosColuna, $nomeColunaDados, $nomeColunaDados, $dadosAgrupador[$nomeColunaDados], $entityContainerColuna);
                    $valoresColunaDados[$key] = $entityContainerColuna->getEvento($nomeColunaDados);
                    unset($entityContainerColuna);
                }
                $this->percorreDadosAgrupador(
                    $nomeEvento, $nomeAgrupador, array($nomeColunaDados => $nomeColunaDados), $nomeColunaDados, $valoresColunaDados, $entityContainer
                );
            } else {
                $this->percorreDadosAgrupador(
                    $nomeEvento, $nomeAgrupador, $dadosAgrupador, $nomeColunaDados, $valorColunaDados, $entityContainer
                );
            }
		}
	}

    /**
     * Percorre todos os dados de uma tabela, dentro de um agrupamento.
     *
     * @param $nomeRegistro       string
     * @param $dadosRegistro
     * @param $nomeColunaDados
     * @param $valorColunaDados 
     * @param $linhaDados
     * @param EntityContainerESocial $entityContainer
     */
    protected function percorreDadosAgrupador($nomeEvento, $nomeAgrupador, $dadosAgrupador, $nomeColunaDados, $valorColunaDados, EntityContainerESocial $entityContainer)
	{
        foreach ($dadosAgrupador as $nomeColuna => $dadosColuna) {
			$this->constroiFieldEntidade(
                $nomeEvento, $nomeAgrupador, $nomeColunaDados, $valorColunaDados, $nomeColuna, $dadosColuna, $entityContainer
			);
		}
	}

	protected function constroiFieldEntidade($nomeEvento, $nomeAgrupador, $nomeColunaDados, $valorColunaDados, $nomeColuna, $dadosColuna, EntityContainerESocial $entityContainer)
	{
        /** @var  $entidade BasicEntityESocial*/
        $entidade = $entityContainer->getEvento($nomeEvento);
        // $nomeColuna      = strtoupper(Util::removeCaracteresInvalidos($nomeColuna));
        // $nomeColunaDados = strtoupper(Util::removeCaracteresInvalidos($nomeColunaDados));

        if ($nomeColuna === $nomeColunaDados) {
			$this->addFields($entidade, $dadosColuna, $valorColunaDados);
		}
	}

	protected function addFields(BasicEntityESocial $entidade, $dadosColuna, $valorColunaDados)
	{
		$fields = explode(",", str_replace(" ", "", $dadosColuna));
		foreach ($fields as $field) {
			$entidade->addField($field, $valorColunaDados);
		}
	}

	protected function addFixedFields(BasicEntityESocial $entidade)
	{
		foreach ($this->fixedFields as $name => $value) {
			if (!$entidade->hasColumn($name)) {
				$entidade->addField($name, $value);
			}
		}
	}

    /**
     * @param EntityContainerESocial $entityContainer
     */
    protected function setEntidades(EntityContainerESocial $entityContainer)
    {
        foreach ($entityContainer->getEventos() as $evento) {
            array_push($this->entidades, $evento);
        }
    }
}