<?php
namespace Folha\Entities;

use HCM\Util\DateUtil;
use Importacao\Logic\Integracao\ProjetoImportacao;

class MigAlteocupacao extends Base\MigAlteocupacao
{

    const EXPRESSAO_NUMEROS = '/[^0-9]/';

    /**
     * @param $nralteocupacao
     * @return \Folha\Entities\MigAlteocupacao
     */
    public function setNralteocupacao ($nralteocupacao) {
        $nralteocupacao = \Importacao\Logic\Validacoes::verificaChaveDuplicada('GPE_ALTEOCUPACAO', 'NRALTEOCUPACAO', ProjetoImportacao::getNrorg(), $nralteocupacao);
        $this->nralteocupacao = $nralteocupacao;
    }

    /**
     * @param $nrvinculom
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setNrvinculom ($nrvinculom) {
        $nrvinculom = \Importacao\Logic\Validacoes::verificaChaveMae('GPE_VINCULOM', 'NRVINCULOM', ProjetoImportacao::getNrorg(), $nrvinculom);
        $this->nrvinculom = $nrvinculom;
    }

    /**
     * @param $nrocupacaom
     * @throws \Exception
     * @return \Folha\Entities\MigAlteocupacao
     */
    public function setNrocupacaom ($nrocupacaom) {
        $nrocupacaom = preg_replace(self::EXPRESSAO_NUMEROS, "", $nrocupacaom);
        $nrorg = ProjetoImportacao::getNrorg();
        $ocupacaom = \Folha\Entities\MigOcupacaom::getObjectByWhere('NROCUPACAOM = ? AND NRORG = ?', array($nrocupacaom, $nrorg));

        if (is_object($ocupacaom)) {
            $this->nrocupacaom = $nrocupacaom;
        } else {
            throw new \Exception("O código da Ocupacao '$nrocupacaom' não foi encontrado.");
        }
    }

    /**
     * @param $dtiniocupacao
     * @throws \Exception
     * @return \Folha\Entities\MigAlteocupacao
     */
    public function setDtiniocupacao ($dtiniocupacao) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtiniocupacao)) {
            throw new \Exception("Campo Obrigatório - Início Ocupação: Data '$dtiniocupacao' com formato inválido ou em branco.");
        }

        if (!($dtiniocupacao instanceof \DateTime)) {
            $dtiniocupacao = DateUtil::getDataDeString($dtiniocupacao);
        }

        $this->dtiniocupacao = $dtiniocupacao;
    }

    /**
     * @param $dtfimocupacao
     * @throws \Exception
     * @return \Folha\Entities\MigAlteocupacao
     */
    public function setDtfimocupacao ($dtfimocupacao) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtfimocupacao)) {
            if (!empty($dtfimocupacao)) {
                throw new \Exception("Dt. Fim Ocupação: Data '$dtfimocupacao' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtfimocupacao instanceof \DateTime)) {
            $dtfimocupacao = DateUtil::getDataDeString($dtfimocupacao);
        }

        $this->dtfimocupacao = $dtfimocupacao;
    }

    /**
     * @param $dtultatu
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setDtultatu ($dtultatu) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtultatu)) {
            throw new \Exception("Data '$dtultatu' com formato inválido ou em branco.");
        }

        if (!($dtultatu instanceof \DateTime)) {
            $dtultatu = DateUtil::getDataDeString($dtultatu);
        }

        $this->dtultatu = $dtultatu;
    }

    /**
     * @param $dtinclusao
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setDtinclusao ($dtinclusao) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtinclusao)) {
            throw new \Exception("Data '$dtinclusao' com formato inválido ou em branco.");
        }

        if (!($dtinclusao instanceof \DateTime)) {
            $dtinclusao = DateUtil::getDataDeString($dtinclusao);
        }

        $this->dtinclusao = $dtinclusao;
    }

}